// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Main.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.applet.Applet;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import jnb.*;

public class Main extends JApplet{

    protected static final String	version = "1.0";

    protected static JnBPanel	panel;
    protected static MenuBar	menuBar;
    protected static StatusBar	statusBar;
    protected static int	windowWidth = 640;
    protected static int	windowHeight = 480;

    public void init(){
	//Try to get window's width and height from applet parameters.
	try {
	    String windowWidthString = getParameter("WIDTH");

	    if (windowWidthString != null) {
		try {
		    windowWidth = Integer.parseInt(windowWidthString);
		} catch (NumberFormatException e) {
		    //Use default width.
		}
	    }
	} catch (NullPointerException e) {
	    //Applet used as a stand-alone program, no parameter.
	}
	try {
	    String windowHeightString = getParameter("HEIGHT");

	    if (windowHeightString != null) {
		try {
		    windowHeight = Integer.parseInt(windowHeightString);
		} catch (NumberFormatException e) {
		    //Use default width.
		}
	    }
	} catch (NullPointerException e) {
	    //Applet used as a stand-alone program, no parameter.
	}

	//Initialize the layout.
        getContentPane().setLayout(new BorderLayout());
        panel = new JnBPanel();
        getContentPane().add(panel);

	//Initialize the Menu bar.
        menuBar = new MenuBar();
        getContentPane().add("North", menuBar);

	//Initialize the Status bar.
        statusBar = new StatusBar("Welcome to Java & Bzier " + version + "!");
        getContentPane().add("South", statusBar);
    }

    public static void main(String s[]) {
        JFrame f = new JFrame("Java & Bzier " + version);
        JApplet applet = new Main();

        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        f.getContentPane().add(applet, BorderLayout.CENTER);
        f.pack();
        f.setSize(new Dimension(windowWidth, windowHeight));
        applet.init();
        f.setVisible(true);
    }
}


class JnBPanel extends JPanel implements MouseListener, MouseMotionListener{

    BufferedImage		bi;
    Graphics2D			big;
    boolean			firstTime = true;
    boolean			pressOut = false;
    boolean			mouseOutside = false;
    boolean			antiAliasing = true;
    Board2D			board;
    protected int		width, height;

    public JnBPanel(){
	
	setBackground(Color.white);
	addMouseMotionListener(this);
	addMouseListener(this);
    }

    public void mousePressed(MouseEvent e){
	int i;

	//BUTTON1 or BUTTON2
	if ((e.getModifiers() & (InputEvent.BUTTON1_MASK | InputEvent.BUTTON2_MASK)) > 0){
	    if (board.isVisible()){
		if ((i = board.contains(e.getX(), e.getY())) >= 0){
		    board.setCurrent(i);

		    Point2DList pl =  (Point2DList)board.getCurrent();

		    pl.setCurrent(pl.contains(e.getX(), e.getY()));
		    Main.statusBar.displayMsg("Point (X:" + e.getX() + ", Y:" + e.getY() + ") selected.");
		    updateLocation(e);
		}
		else {
		    if (board.size() == 0 || (e.getModifiers() & InputEvent.BUTTON2_MASK) > 0){
			Point2DList pl = new Point2DList();
			
			board.add(pl);
		    }
		    Point2DList pl =  (Point2DList)board.getCurrent();
		    jnb.Point2D point = new jnb.Point2D(e.getX(), e.getY());
		    
		    pl.add(pl.getCurrentIndex() + 1, point);
		    Main.statusBar.displayMsg("Point (X:" + e.getX() + ", Y:" + e.getY() + ") added.");
		}
	    }
	    else {
		pressOut = true;	
	    }
	}

	//BUTTON3
	else if ((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0){
	    if(board.isVisible()){
		if ((i = board.contains(e.getX(), e.getY())) >= 0){
		    Point2DList pl =  (Point2DList)board.get(i);
		    if (pl.remove(pl.contains(e.getX(), e.getY())) == 1)
			board.remove(pl);
		    else
			board.setActive(pl);
		    Main.statusBar.displayMsg("Point (X:" + e.getX() + ", Y:" + e.getY() + ") removed.");
		    repaint();
		}
	    }
	}
    }
    
    public void mouseDragged(MouseEvent e){
	if((e.getModifiers() & (InputEvent.BUTTON1_MASK | InputEvent.BUTTON2_MASK)) > 0 && !mouseOutside && !pressOut) {
	    updateLocation(e);
	}
    }

    public void mouseReleased(MouseEvent e){
	if(board.isVisible() && ((e.getModifiers() & InputEvent.BUTTON3_MASK) == 0)){
	    updateLocation(e);
	}
	else {
	    pressOut = false;
	}
    }

    public void mouseMoved(MouseEvent e){
	Main.statusBar.displayMsgBG("(X:" + e.getX() + ", Y:" + e.getY() + ")");
    }

    public void mouseExited(MouseEvent e){
	mouseOutside = true;
    }

    public void mouseEntered(MouseEvent e){
	mouseOutside = false;
    }

    public void mouseClicked(MouseEvent e){}

    public void updateLocation(MouseEvent e){
	Dimension dim = getSize();
	Point2DList pl =  (Point2DList)board.getCurrent();

	if (pl == null)
	    return;
	
	jnb.Point2D point = (jnb.Point2D)pl.getCurrent();
	int x, y;

	if (board.getGrid()){
	    x = jnb.Math.round(e.getX(), board.getGridSpacing());
	    y = jnb.Math.round(e.getY(), board.getGridSpacing());
	} else {
	    x = e.getX();
	    y = e.getY();
	}
	Main.statusBar.displayMsgBG("Point moved to (X:" + x + ", Y:" + y + ").");
	if ((e.getModifiers() & InputEvent.BUTTON1_MASK) > 0)
	    point.setLocation(x, y, dim);
	else if ((e.getModifiers() & InputEvent.BUTTON2_MASK) > 0)
	    pl.setLocation(x, y, dim);
	repaint();
    }

    public boolean isAntiAliased() {
	return antiAliasing;
    }

    public void setAntiAliasing(boolean aa) {
	antiAliasing = aa;
	if (antiAliasing)
	    big.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	else
	    big.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);	
    }

    public void paintComponent(Graphics g){
	super.paintComponent(g);
	update(g);
    }
    
    public void update(Graphics g){
	Dimension dim = getSize();
	int w = dim.width;
	int h = dim.height;
	
	if(firstTime){
	    width = w;
	    height = h;
	    board = new Board2D(dim);
	    bi = (BufferedImage)createImage(w, h);
	    big = bi.createGraphics();
	    big.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	    big.setColor(Color.white);
	    firstTime = false;
	}
	if (w != width || h != height) {
	    width = w;
	    height = h;
	    board.setDimension(dim);
	    bi = (BufferedImage)createImage(w, h);
	    big = bi.createGraphics();
	    big.setColor(Color.white);
	}
	
	// Clears the drawing area.
	big.clearRect(0, 0, w, h);
	
	// Draws and fills the points and curve(s) on the board.
	board.draw(big);

	// Draws the buffered image to the screen.
	g.drawImage(bi, 0, 0, this);
    }
}
