// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// MenuBar.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import jnb.*;

public class MenuBar extends JMenuBar{
    JMenu menu, subMenu;
    JMenuItem menuItem;
    JCheckBox toggleGrid, ptsDisp, antiAlias;
    
    public MenuBar() {
	super();

	ItemListener il = new MenuBarItemListener();
	ActionListener al = new MenuBarActionListener();

	// Edit menu
	menu = new JMenu("Edit");
	menu.setMnemonic(KeyEvent.VK_E);
	add(menu);
	
	menuItem = new JMenuItem("Remove current Point", KeyEvent.VK_C);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("rm_point");
	menu.add(menuItem);

	menuItem = new JMenuItem("Remove Curves of current Set of Points", KeyEvent.VK_M);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("rm_curves");
	menu.add(menuItem);

	menuItem = new JMenuItem("Remove current Set of Points", KeyEvent.VK_R);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("rm_points");
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem("Select Next Point", KeyEvent.VK_X);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("next_point");
	menu.add(menuItem);

	menuItem = new JMenuItem("Select Previous Point", KeyEvent.VK_V);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("prev_point");
	menu.add(menuItem);

	menuItem = new JMenuItem("Select Next Set of Points", KeyEvent.VK_N);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("next_points");
	menu.add(menuItem);

	menuItem = new JMenuItem("Select Previous Set of Points", KeyEvent.VK_P);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("prev_points");
	menu.add(menuItem);
	
	menu.addSeparator();

	menuItem = new JMenuItem("Clear", KeyEvent.VK_C);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("clear");
	menu.add(menuItem);

	// Curves menu
	menu = new JMenu("Curves");
	menu.setMnemonic(KeyEvent.VK_C);
	add(menu);

	menuItem = new JMenuItem("Draw Polygon", KeyEvent.VK_P);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("polygon");
	menu.add(menuItem);

	subMenu = new JMenu("Interpolation");
	subMenu.setMnemonic(KeyEvent.VK_I);
	menu.add(subMenu);

	menuItem = new JMenuItem("Draw Lagrange Curve", KeyEvent.VK_L);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("lagrange");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw 3-degree Hermite Curve", KeyEvent.VK_H);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("hermite");
	subMenu.add(menuItem);

	subMenu = new JMenu("Conics");
	subMenu.setMnemonic(KeyEvent.VK_C);
	menu.add(subMenu);

	menuItem = new JMenuItem("Draw Circle", KeyEvent.VK_C);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("circle");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw Ellipse", KeyEvent.VK_E);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("ellipse");
	subMenu.add(menuItem);

	subMenu.addSeparator();

	menuItem = new JMenuItem("Approximate Circle with Bzier Curve", KeyEvent.VK_A);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("circlebzier");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Approximate Ellipse with Bzier Curve", KeyEvent.VK_B);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("ellipsebzier");
	subMenu.add(menuItem);

	subMenu = new JMenu("Bzier");
	subMenu.setMnemonic(KeyEvent.VK_B);
	menu.add(subMenu);

	menuItem = new JMenuItem("Draw 2-degree Bzier Curve", KeyEvent.VK_2);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bzier2");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw 3-degree Bzier Curve", KeyEvent.VK_3);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bzier3");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw n-degree Bzier Curve", KeyEvent.VK_B);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bzier");
	subMenu.add(menuItem);

	subMenu = new JMenu("Uniform B-Spline");
	subMenu.setMnemonic(KeyEvent.VK_S);
	menu.add(subMenu);

	menuItem = new JMenuItem("Draw 2-degree B-Spline Curve", KeyEvent.VK_B);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bspline2");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw 3-degree B-Spline Curve", KeyEvent.VK_S);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bspline3");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Draw n-degree B-Spline Curve", KeyEvent.VK_N);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bspline");
	subMenu.add(menuItem);

	subMenu.addSeparator();

	menuItem = new JMenuItem("Convert 2-degree B-Spline Curve to Bzier Curve", KeyEvent.VK_2);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bspline2bzier");
	subMenu.add(menuItem);

	menuItem = new JMenuItem("Convert 3-degree B-Spline Curve to Bzier Curve", KeyEvent.VK_3);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("bspline3bzier");
	subMenu.add(menuItem);

	// NURBS: WIP...
	//subMenu = new JMenu("NURBS");
	//subMenu.setMnemonic(KeyEvent.VK_N);
	//menu.add(subMenu);

	// Options menu
	menu = new JMenu("Options");
	menu.setMnemonic(KeyEvent.VK_O);
	add(menu);

	ptsDisp = new JCheckBox("Points display");
	ptsDisp.setMnemonic(KeyEvent.VK_P);
	ptsDisp.setSelected(true);
	ptsDisp.addItemListener(il);
	menu.add(ptsDisp);

	menu.addSeparator();

	toggleGrid = new JCheckBox("Grid");
	toggleGrid.setMnemonic(KeyEvent.VK_G);
	toggleGrid.setSelected(false);
	toggleGrid.addItemListener(il);
	menu.add(toggleGrid);

	menuItem = new JMenuItem("Increase Grid spacing", KeyEvent.VK_I);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("inc_grid");
	menu.add(menuItem);

	menuItem = new JMenuItem("Decrease Grid spacing", KeyEvent.VK_D);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("dec_grid");
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem("Increase Curves detail", KeyEvent.VK_N);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("inc_detail");
	menu.add(menuItem);

	menuItem = new JMenuItem("Decrease Curves detail", KeyEvent.VK_E);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, ActionEvent.ALT_MASK));
	menuItem.addActionListener(al);
	menuItem.setActionCommand("dec_detail");
	menu.add(menuItem);

	menu.addSeparator();

	antiAlias = new JCheckBox("Anti-aliasing");
	antiAlias.setMnemonic(KeyEvent.VK_A);
	antiAlias.setSelected(true);
	antiAlias.addItemListener(il);
	menu.add(antiAlias);

	// Help menu
	menu = new JMenu("Help");
	menu.setMnemonic(KeyEvent.VK_H);
	add(menu);

	menuItem = new JMenuItem("Manual", KeyEvent.VK_M);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("manual");
	menu.add(menuItem);
	
	menu.addSeparator();

	menuItem = new JMenuItem("About", KeyEvent.VK_A);
	menuItem.addActionListener(al);
	menuItem.setActionCommand("about");
	menu.add(menuItem);
    }

    class MenuBarActionListener implements ActionListener{

	public void actionPerformed(ActionEvent e) {
	    // Edit menu
	    if (e.getActionCommand().equals("rm_point")) {
		if (Main.panel.board.isVisible()){
		    PointList pl;

		    if ((pl = Main.panel.board.getCurrent()) != null)
			if (pl.remove(pl.getCurrent()) == 1)
			    Main.panel.board.remove(pl);
		    Main.statusBar.displayMsg("Current Point removed.");
		}
	    } else if (e.getActionCommand().equals("rm_curves")) {
		if (Main.panel.board.isVisible()){
		    PointList pl;
		    CurveList cl;
		    
		    pl = Main.panel.board.getCurrent();
		    cl = (CurveList)Main.panel.board.getCurveList();
		    cl.remove(pl);
		    Main.statusBar.displayMsg("Curves of the current Set of Points removed.");
		}
	    } else if (e.getActionCommand().equals("rm_points")) {
		if (Main.panel.board.isVisible()){
		    PointList pl;
		    CurveList cl;

		    pl = Main.panel.board.getCurrent();
		    cl = (CurveList)Main.panel.board.getCurveList();
		    cl.remove(pl);
		    Main.panel.board.remove(pl);
		    Main.statusBar.displayMsg("Current Set of Points removed.");
		}
	    } else if (e.getActionCommand().equals("next_point")) {
		if (Main.panel.board.isVisible()){
		    if (Main.panel.board.size() > 0) {
			PointList pl = Main.panel.board.getCurrent();
			Point2D p;
			pl.setCurrent(jnb.Math.min(pl.getCurrentIndex() + 1, pl.size() - 1));
			p = (Point2D)pl.getCurrent();
			Main.statusBar.displayMsg("Next Point (X:" + (int)p.getX() + ", Y:" + (int)p.getY() + ") selected.");
		    }
		}
	    } else if (e.getActionCommand().equals("prev_point")) {
		if (Main.panel.board.isVisible()){
		    if (Main.panel.board.size() > 0) {
			PointList pl = Main.panel.board.getCurrent();
			Point2D p;
			pl.setCurrent(jnb.Math.max(pl.getCurrentIndex() - 1, 0));
			p = (Point2D)pl.getCurrent();
			Main.statusBar.displayMsg("Previous Point (X:" + (int)p.getX() + ", Y:" + (int)p.getY() + ") selected.");
		    }
		}
	    } else if (e.getActionCommand().equals("next_points")) {
		if (Main.panel.board.isVisible()){
		    if (Main.panel.board.size() > 0) {
			Main.panel.board.setCurrent(jnb.Math.min(Main.panel.board.getCurrentIndex() + 1, Main.panel.board.size() - 1));
			Main.statusBar.displayMsg("Next Set of Points selected.");
		    }
		}
	    } else if (e.getActionCommand().equals("prev_points")) {
		if (Main.panel.board.isVisible()){
		    if (Main.panel.board.size() > 0) {
			Main.panel.board.setCurrent(jnb.Math.max(Main.panel.board.getCurrentIndex() - 1, 0));
			Main.statusBar.displayMsg("Previous Set of Points selected.");
		    }
		}
	    } else if (e.getActionCommand().equals("clear")) {
		Main.panel.board.clear();
		Main.statusBar.displayMsg("Screen cleared.");
	    }
	    
	    // Curves menu
	    else if (e.getActionCommand().equals("polygon")) {
		CurveList cl;
		Curve2D p = new Curve2DPolygon((Point2DList)Main.panel.board.getCurrent());
		
		cl = Main.panel.board.getCurveList();
		p.setColor(Color.lightGray);
		cl.add(p);
		Main.statusBar.displayMsg("Polygon drawn.");
	    } else if (e.getActionCommand().equals("hermite")) {
		CurveList cl;
		Curve2D l = new Curve2DHermite((Point2DList)Main.panel.board.getCurrent());
		
		cl = Main.panel.board.getCurveList();
		l.setColor(new Color(0, 0, 150));
		cl.add(l);
		Main.statusBar.displayMsg("3-degree Hermite Interpolation Curve drawn.");
	    } else if (e.getActionCommand().equals("lagrange")) {
		CurveList cl;
		Curve2D l = new Curve2DLagrange((Point2DList)Main.panel.board.getCurrent());
		
		cl = Main.panel.board.getCurveList();
		l.setColor(new Color(0, 0, 255));
		cl.add(l);
		Main.statusBar.displayMsg("Lagrange Interpolation Curve drawn.");
	    } else if (e.getActionCommand().equals("circle")) {
		CurveList cl;
		Curve2D l = new Curve2DEllipse(true, (Point2DList)Main.panel.board.getCurrent());
		
		cl = Main.panel.board.getCurveList();
		l.setColor(new Color(0, 200, 200));
		cl.add(l);
		Main.statusBar.displayMsg("Circle drawn.");
	    } else if (e.getActionCommand().equals("ellipse")) {
		CurveList cl;
		Curve2D l = new Curve2DEllipse(false, (Point2DList)Main.panel.board.getCurrent());
		
		cl = Main.panel.board.getCurveList();
		l.setColor(new Color(0, 255, 255));
		cl.add(l);
		Main.statusBar.displayMsg("Ellipse drawn.");
	    } else if (e.getActionCommand().equals("circlebzier")) {
		Curve2D b;

		if ((b = Curve2DConv.EllipseToBezier(true, Main.panel.board)) != null) {
		    b.setColor(new Color(0, 150, 0));
		    Main.statusBar.displayMsg("Circle converted to Bezier Curve.");
		}
	    } else if (e.getActionCommand().equals("ellipsebzier")) {
		Curve2D b;
		
		if ((b = Curve2DConv.EllipseToBezier(false, Main.panel.board)) != null) {
		    b.setColor(new Color(0, 200, 0));
		    Main.statusBar.displayMsg("Ellipse converted to Bezier Curve.");
		}
	    } else if (e.getActionCommand().equals("bzier2")) {
		Curve2DList cl;
		Curve2D b = new Curve2DBezierFixed((Point2DList)Main.panel.board.getCurrent(), 2);

		cl = (Curve2DList)Main.panel.board.getCurveList();
		b.setColor(new Color(0, 150, 0));
		cl.add(b);
		Main.statusBar.displayMsg("2-degree Bzier Curve drawn.");
	    } else if (e.getActionCommand().equals("bzier3")) {
		Curve2DList cl;
		Curve2D b = new Curve2DBezierFixed((Point2DList)Main.panel.board.getCurrent(), 3);

		cl = (Curve2DList)Main.panel.board.getCurveList();
		b.setColor(new Color(0, 200, 0));
		cl.add(b);
		Main.statusBar.displayMsg("3-degree Bzier Curve drawn.");
	    } else if (e.getActionCommand().equals("bzier")) {
		if (Main.panel.board.getCurrent() != null) {
		    Curve2DList cl;
		    Curve2D b = new Curve2DBezier((Point2DList)Main.panel.board.getCurrent());

		    cl = (Curve2DList)Main.panel.board.getCurveList();
		    b.setColor(new Color(0, 255, 0));
		    cl.add(b);
		    Main.statusBar.displayMsg(Main.panel.board.getCurrent().size() - 1 +
					      "-degree Bzier Curve drawn.");
		}
	    } else if (e.getActionCommand().equals("bspline2")) {
		Curve2DList cl;
		Curve2D s = new Curve2DBSpline((Point2DList)Main.panel.board.getCurrent(), 2);
		
		cl = (Curve2DList)Main.panel.board.getCurveList();
		s.setColor(new Color(150, 0, 150));
		cl.add(s);
		Main.statusBar.displayMsg("2-degree B-Spline drawn.");
	    } else if (e.getActionCommand().equals("bspline3")) {
		Curve2DList cl;
		Curve2D s = new Curve2DBSpline((Point2DList)Main.panel.board.getCurrent(), 3);
		
		cl = (Curve2DList)Main.panel.board.getCurveList();
		s.setColor(new Color(200, 0, 200));
		cl.add(s);
		Main.statusBar.displayMsg("3-degree B-Spline drawn.");
	    } else if (e.getActionCommand().equals("bspline")) {
		Curve2DList cl;
		Curve2D s = new Curve2DBSpline((Point2DList)Main.panel.board.getCurrent());
		
		cl = (Curve2DList)Main.panel.board.getCurveList();
		s.setColor(new Color(255, 0, 255));
		cl.add(s);
		Main.statusBar.displayMsg(Main.panel.board.getCurrent().size() - 1 +
					  "-degree B-Spline drawn.");
	    } else if (e.getActionCommand().equals("bspline2bzier")) {
		Curve2D b;

		if ((b = Curve2DConv.BSplineToBezier(Main.panel.board, 2)) != null) {
		    b.setColor(new Color(0, 150, 0));
		    Main.statusBar.displayMsg("2-degree B-Spline Curve converted to Bezier Curve.");
		}
	    } else if (e.getActionCommand().equals("bspline3bzier")) {
		Curve2D b;

		if ((b = Curve2DConv.BSplineToBezier(Main.panel.board, 3)) != null) {
		    b.setColor(new Color(0, 200, 0));
		    Main.statusBar.displayMsg("3-degree B-Spline Curve converted to Bezier Curve.");
		}
	    }

	    // Options menu
	    else if (e.getActionCommand().equals("inc_grid")) {
		if (Main.panel.board.getGrid()) {
		    Main.panel.board.setGridSpacing(Main.panel.board.getGridSpacing() + 5);
		    Main.statusBar.displayMsg("Grid spacing increased to " + Main.panel.board.getGridSpacing() + " pixels.");
		}
	    } else if (e.getActionCommand().equals("dec_grid")) {
		if (Main.panel.board.getGrid()) {
		    Main.panel.board.setGridSpacing(Main.panel.board.getGridSpacing() - 5);
		    Main.statusBar.displayMsg("Grid spacing decreased to " + Main.panel.board.getGridSpacing() + " pixels.");
		}
	    } else if (e.getActionCommand().equals("inc_detail")) {
		Curve2D.setSubdiv(Curve2D.getSubdiv() + 1);
		Main.statusBar.displayMsg("Curves detailed increased to " + Curve2D.getSubdiv() + " subdivisions.");
	    } else if (e.getActionCommand().equals("dec_detail")) {
		Curve2D.setSubdiv(Curve2D.getSubdiv() - 1);
		Main.statusBar.displayMsg("Curves detailed decreased to " + Curve2D.getSubdiv() + " subdivisions.");
	    }

	    // Help menu
	    else if (e.getActionCommand().equals("manual")) {
		Main.statusBar.displayMsg("See http://www.lrde.epita.fr/~ender/jnb.");
	    } else if (e.getActionCommand().equals("about")) {
		Main.statusBar.displayMsg("Java & Bzier " +  Main.version + ": created by Emmanuel Turquin <manu@lrde.epita.fr>.");
	    }

	    Main.panel.repaint();
	}
    }

    class MenuBarItemListener implements ItemListener{

	public void itemStateChanged(ItemEvent e) {
	    Object source = e.getItemSelectable();

	    // Options menu
	    if (source == toggleGrid) {
		if (Main.panel.board.getGrid()){
		    Main.panel.board.setGrid(false);
		    Main.statusBar.displayMsg("Grid off.");

		} else {
		    Main.panel.board.setGrid(true);
		    Main.statusBar.displayMsg("Grid on.");
		}
	    }
	    if (source == ptsDisp) {
		if (Main.panel.board.isVisible()) {
		    Main.panel.board.setVisible(false);
		    Main.statusBar.displayMsg("Points display off.");
		} else {
		    Main.panel.board.setVisible(true);
		    Main.statusBar.displayMsg("Points display on.");
		}
	    }
	    if (source == antiAlias) {
		if (Main.panel.isAntiAliased()) {
		    Main.panel.setAntiAliasing(false);
		    Main.statusBar.displayMsg("Anti-aliasing off.");
		} else {
		    Main.panel.setAntiAliasing(true);
		    Main.statusBar.displayMsg("Anti-aliasing on.");
		}
	    }
	    Main.panel.repaint();
	}
    }
}
