// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Board.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package jnb;

import java.util.*;
import java.awt.*;

public abstract class Board{

    java.util.List	       		list;
    CurveList				curveList;
    boolean				visible = true;
    PointList				current;
    PointList				active;
    Color				colorBG = Color.black;
    Color				colorSel = Color.orange;
    Color				colorPointSel = Color.red;

    //constructor
    public Board(){
	list = new LinkedList();
    }

    //listIterator
    public ListIterator listIterator(){
	return list.listIterator();
    }

    //add
    public void add(PointList pl){
	list.add(pl);
	setCurrent(pl);
    }

    public void add(int i, PointList pl){
	list.add(i, pl);
	setCurrent(pl);
    }

    //remove
    public int remove(PointList pl){
	boolean cr = false;
	int i = getCurrentIndex();

	if (pl == current)
	    cr = true;
	list.remove(pl);
	if (size() == 0) {
	    current = null;
	    return 1;
	}
	if (cr)
	    setCurrent(size() - 1);
	return 0;
    }

    public int remove(int i){
	boolean cr = false;

	if (get(i) == current)
	    cr = true;
	list.remove(i);
	if (size() == 0) {
	    current = null;
	    return 1;
	}
	if (cr)
	    setCurrent(size() - 1);
	return 0;
    }

    //clear
    public void clear()
    {
	current = null;
	list.clear();
	curveList.clear();
    }

    //get
    public PointList get(int i){
	return (PointList)list.get(i);
    }

    //size
    public int size(){
	return list.size();
    }

    //getCurveList
    public CurveList getCurveList(){
	return curveList;
    }

    //getCurrent
    public PointList getCurrent(){
	return current;
    }

    //getCurrentIndex
    public int getCurrentIndex(){
	if (current == null)
	    return -1;
	return list.indexOf(current);
    }

    //setCurrent
    public void setCurrent(PointList pl){
	current = pl;
	refreshColor();
    }

    public void setCurrent(int i){
	current = (PointList)list.get(i);
	refreshColor();
    }

    //getActive
    public PointList getActive(){
	return active;
    }

    //setActive
    public void setActive(PointList pl){
	active = pl;
    }

    //getColorBG
    public Color getColorBG(){
	return colorBG;
    }

    //getColorSel
    public Color getColorSel(){
	return colorSel;
    }

    //getColorPointSel
    public Color getColorPointSel(){
	return colorSel;
    }

    //setColor
    public void setColor(Color bg, Color sel, Color ptsel){
	colorBG = bg;
	colorSel = sel;
	colorPointSel = ptsel;
    }

    //isVisible
    public boolean isVisible(){
	return visible;
    }

    //setVisible
    public void setVisible(boolean b){
	PointList pl;

	for (ListIterator li = list.listIterator(); li.hasNext(); )
	    {
		pl = (PointList)li.next();
		pl.setVisible(b);
	    }
	visible = b;
    }

    //refreshColor
    public void refreshColor(){
	PointList pl;

	for (ListIterator li = list.listIterator(); li.hasNext(); )
	    {
		pl = (PointList)li.next();
		if (pl == current)
		    pl.setColor(colorSel, colorPointSel);
		else
		    pl.setColor(colorBG, colorBG);
		pl.refreshColor();
	    }
    }

}
