// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Board2D.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package jnb;
 
import java.util.*;
import java.awt.*;
import java.awt.geom.*;

public class Board2D extends Board{

    static final int		GRID_MIN_SPACING = 10;
    static final float		GRID_THICKNESS = 1.0f;

    Dimension		dim;
    boolean		grid = false;
    int			gridSpacing = 20;
    Color		colorGrid = new Color(230, 230, 230);

    //constructor
    public Board2D(Dimension d){
	super();
	curveList = new Curve2DList();
	dim = d;
    }

    //getDimension
    public Dimension getDimension(){
	return dim;
    }

    //setDimension
    public void setDimension(Dimension d){
	dim = d;
    }

    //contains
    public int contains(double x, double y){
	if (current != null && ((Point2DList)current).contains(x, y) >= 0)
	    return list.indexOf(current);
	Point2DList pl;
	for (ListIterator li = list.listIterator(list.size()); li.hasPrevious(); )
	    {
		pl = (Point2DList)li.previous();
		if (pl.contains(x, y) >= 0)
		    return li.nextIndex();
	    }
	return -1;
    }

    //getGrid
    public boolean getGrid(){
	return grid;
    }

    //setGrid
    public void setGrid(boolean b){
	grid = b;
    }

    //getGridSpacing
    public int getGridSpacing(){
	return gridSpacing;
    }

    //setGridSpacing
    public void setGridSpacing(int gs){
	if (gs <= GRID_MIN_SPACING)
	    gs = GRID_MIN_SPACING;
	else if (gs > java.lang.Math.min(dim.width / 2, dim.height / 2))
	    gs = java.lang.Math.min(dim.width / 2, dim.height / 2);
	gridSpacing = gs;
    }

    //getColorGrid
    public Color getColorGrid(){
	return colorGrid;
    }

    //setColorGrid
    public void setColorGrid(Color cg){
	colorGrid = cg;
    }

    //draw
    public void draw(Graphics2D g){
	Curve2DList cl = (Curve2DList)curveList;
	Point2DList pl;

	if (grid)
	    drawGrid(g);
	cl.draw(g, (Point2DList)current, (Point2DList)active);
	if (active != null)
	    active = null;
	for (ListIterator li = list.listIterator(); li.hasNext(); )
	    {
		pl = (Point2DList)li.next();
		pl.draw(g);
	    }
	if (current != null)
	    ((Point2DList)current).draw(g);
    }

    //drawGrid
    void drawGrid(Graphics2D g){
	Line2D.Double line = new Line2D.Double();

	g.setPaint(colorGrid);
	g.setStroke(new BasicStroke(GRID_THICKNESS));
	for (int i = gridSpacing; i < dim.width; i += gridSpacing)
	    {
		line.setLine(i, 0, i, dim.height);
		g.draw(line);
	    }
	for (int i = gridSpacing; i < dim.height; i += gridSpacing)
	    {
		line.setLine(0, i, dim.width, i);
		g.draw(line);
	    }
    }
}
