// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Curve2DBSpline.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package jnb;

import java.util.*;
import java.awt.*;
import java.lang.*;

public class Curve2DBSpline extends Curve2D{

    static final int MAX_DEGREE = 15;

    double[]	tmp;
    double	fact[];
    int		n;
    int		m = 1;
    int		size;
    boolean	fixedDegree = false;

    //constructor
    public Curve2DBSpline(){
	super();
	nbMin = m + 1;
    }

    public Curve2DBSpline(int degree){
	super();
	m = Math.max(degree, 2);
	nbMin = m + 1;
	fact = new double[m + 2];
	for (int i = 0; i <= m + 1; i++)
	    fact[i] = Math.fact(i);
	fixedDegree = true;
    }

    public Curve2DBSpline(Point2DList pl){
	super(pl);
	nbMin = m + 1;
    }

    public Curve2DBSpline(Point2DList pl, int degree){
	super(pl);
	m = Math.max(degree, 2);
	nbMin = m + 1;
	fact = new double[m + 2];
	for (int i = 0; i <= m + 1; i++)
	    fact[i] = Math.fact(i);
	fixedDegree = true;
  }

    //computeSegments
    void computeSegments(){
	Point2DList pl = (Point2DList)plist;

	if (!fixedDegree) {
	    m = Math.min(pl.size() - 1, MAX_DEGREE);
	    fact = new double[m + 2];
	    for (int i = 0; i <= m + 1; i++)
		fact[i] = Math.fact(i);
	}
	n = pl.size() - m;

	size = subdiv * m;
	tmp = new double[size + 1];
	for (int i = 0; i <= size; i++)
	    tmp[i] = (double)i / size;

	java.awt.geom.Point2D.Double p;
	double x, y, bt;



	for (int k = 0; k <= n - 1; k++)
	    {
		for (int t = 0; t <= size; t++) {
		    x = 0;
		    y = 0;
		    for (int i = 0; i <= m; i++) {
			bt = BSplineTerm(i, t, fact);
			x += ((Point2D)pl.get(i + k)).getX() * bt;
			y += ((Point2D)pl.get(i + k)).getY() * bt;
		    }
		    p = new java.awt.geom.Point2D.Double(x, y);
		    list.add(p);
		}
	    }
    }

    double BSplineTerm(int i, int t, double[] fact){
	double d = 0;

	for (int k = 0; k <= m - i; k++)
	    d += Math.pow(-1, k) * Math.pow(tmp[t] + m - i - k, m) / (fact[k] * fact[m - k + 1]);
	return (double)(m + 1) * d;
    }
    
}
