// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Curve2DBezier.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package jnb;

import java.util.*;
import java.awt.*;
import java.lang.*;

public class Curve2DBezier extends Curve2D{

    double[]	tmp;
    double	time = 1;
    int		size;
    int		n;

    //constructor
    public Curve2DBezier(){
	super();
    }

    public Curve2DBezier(Point2DList pl){
	super(pl);
    }

    public Curve2DBezier(Point2DList pl, double t){
	super(pl);
	time = t;
    }

    //getTime
    public double getTime() {
	return time;
    }

    //setTime
    public void setTime(double t) {
	time = Math.min(Math.max(t, 0), 1);
    }

    //computeSegments
    void computeSegments(){
	Point2DList pl = (Point2DList)plist;

	n = pl.size() - 1;

	size = (int)(subdiv * n * time);
	tmp = new double[size + 1];
	for (int i = 0; i <= size ; i++)
	    tmp[i] = (double)time * i / size;

	java.awt.geom.Point2D.Double p;
	double x, y, bt;

	for (int t = 0; t <= size; t++) {
	    x = 0;
	    y = 0;
	    for (int i = 0; i <= n; i++) {
		bt = BezierTerm(i, t);
		x += ((Point2D)pl.get(i)).getX() * bt;
		y += ((Point2D)pl.get(i)).getY() * bt;
	    }
	    p = new java.awt.geom.Point2D.Double(x, y);
	    list.add(p);
	}
    }

    double BezierTerm(int i, int t){
	return (double)(Math.comb(n, i) * Math.pow(1 - tmp[t], n - i) * Math.pow(tmp[t], i));
    }
    
}
