// Java & Bzier
// (C) 2002 Emmanuel Turquin
//
// Curve2DConv.java
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package jnb;

import java.awt.*;

public final class Curve2DConv{
    
    public static Curve2D EllipseToBezier(boolean circle, Board2D board){
	Point2DList pl = (Point2DList)board.getCurrent();

	if (pl == null)
	    return null;

	int s = pl.size();
	
	if (s >= 2){
	    Point2DList plnew = new Point2DList();
	    Point2D p;
	    CurveList cl = board.getCurveList();
	    Curve2D b;

	    double x, y, x0, y0, x1, y1, c;

	    c = 4 * (java.lang.Math.sqrt(2) - 1) / 3;
	    x0 = ((Point2D)pl.get(0)).getX();
	    y0 = ((Point2D)pl.get(0)).getY();
	    x1 = ((Point2D)pl.get(1)).getX();
	    y1 = ((Point2D)pl.get(1)).getY();
	    if (circle) {
		x = y = java.lang.Math.pow(java.lang.Math.pow(x1 - x0, 2) +
					   java.lang.Math.pow(y1 - y0, 2), 0.5);
	    } else {
		x = x1 - x0;
		y = y0 - y1;
	    }

	    p = new Point2D(x0 + x, y0);
	    plnew.add(p);
	    p = new Point2D(x0 + x, y0 + y * c);
	    plnew.add(p);
	    p = new Point2D(x0 + x * c, y0 + y);
	    plnew.add(p);
	    p = new Point2D(x0, y0 + y);
	    plnew.add(p);
	    p = new Point2D(x0 - x * c, y0 + y);
	    plnew.add(p);
	    p = new Point2D(x0 - x, y0 + y * c);
	    plnew.add(p);
	    p = new Point2D(x0 - x, y0);
	    plnew.add(p);
	    p = new Point2D(x0 - x, y0 - y * c);
	    plnew.add(p);
	    p = new Point2D(x0 - x * c, y0 - y);
	    plnew.add(p);
	    p = new Point2D(x0, y0 - y);
	    plnew.add(p);
	    p = new Point2D(x0 + x * c, y0 - y);
	    plnew.add(p);
	    p = new Point2D(x0 + x, y0 - y * c);
	    plnew.add(p);
	    p = new Point2D(x0 + x, y0);
	    plnew.add(p);



	    board.add(plnew);
	    b = new Curve2DBezierFixed(plnew, 3);
	    cl.add(b);
	    return b;
	}
	return null;
    }
    
    public static Curve2D BSplineToBezier(Board2D board, int deg){
	Point2DList pl = (Point2DList)board.getCurrent();

	if (pl == null)
	    return null;

	int s = pl.size();
	
	if (deg == 2 && s >= 3){
	    Point2DList plnew = new Point2DList();
	    Point2D p;
	    CurveList cl = board.getCurveList();
	    Curve2D b;

	    p = new Point2D((((Point2D)pl.get(0)).getX() + ((Point2D)pl.get(1)).getX()) / 2,
			    (((Point2D)pl.get(0)).getY() + ((Point2D)pl.get(1)).getY()) / 2);
	    plnew.add(p);
	    for (int i = 0; i <= s - 3; i++){
		p = new Point2D((Point2D)pl.get(i + 1));
		plnew.add(p);
		p = new Point2D((((Point2D)pl.get(i + 1)).getX() + ((Point2D)pl.get(i + 2)).getX()) / 2,
				(((Point2D)pl.get(i + 1)).getY() + ((Point2D)pl.get(i + 2)).getY()) / 2);
		plnew.add(p);
	    }
	    board.add(plnew);
	    b = new Curve2DBezierFixed(plnew, 2);
	    cl.add(b);
	    return b;
	}
	else if (deg == 3 && s >= 4) {
	    Point2DList plnew = new Point2DList();
	    Point2D p;
	    CurveList cl = board.getCurveList();
	    Curve2D b;

	    p = new Point2D((((Point2D)pl.get(0)).getX() + 4 * ((Point2D)pl.get(1)).getX() + ((Point2D)pl.get(2)).getX()) / 6,
			    (((Point2D)pl.get(0)).getY() + 4 * ((Point2D)pl.get(1)).getY() + ((Point2D)pl.get(2)).getY()) / 6);
	    plnew.add(p);
	    for (int i = 0; i <= s - 4; i++){
		p = new Point2D((2 * ((Point2D)pl.get(i + 1)).getX() + ((Point2D)pl.get(i + 2)).getX()) / 3,
				(2 * ((Point2D)pl.get(i + 1)).getY() + ((Point2D)pl.get(i + 2)).getY()) / 3);
		plnew.add(p);
		p = new Point2D((((Point2D)pl.get(i + 1)).getX() + 2 * ((Point2D)pl.get(i + 2)).getX()) / 3,
				(((Point2D)pl.get(i + 1)).getY() + 2 * ((Point2D)pl.get(i + 2)).getY()) / 3);
		plnew.add(p);
		p = new Point2D((((Point2D)pl.get(i + 1)).getX() + 4 * ((Point2D)pl.get(i + 2)).getX() +
				 ((Point2D)pl.get(i + 3)).getX()) / 6,
				(((Point2D)pl.get(i + 1)).getY() + 4 * ((Point2D)pl.get(i + 2)).getY() +
				 ((Point2D)pl.get(i + 3)).getY()) / 6);
		plnew.add(p);
	    }
	    board.add(plnew);
	    b = new Curve2DBezierFixed(plnew, 3);
	    cl.add(b);
	    board.setVisible(board.isVisible());
	    return b;
	}
	return null;
    }
    
}
